==> lvm-0.3alpha.tar.gz: LVM/0.3alpha/tools/lvreduce.c <==
/*
 * Author: Heinz Mauelshagen, Germany
 *
 * April 1997
 * April,May 1998
 *
 * LVM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * LVM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 */

/*
 * Changelog
 *
 *    10/16/1997 - change lv_get_index to lv_get_index_by_name
 *    11/09/1097 - added lvmtab handling
 *    04/30/1998 - changed to lv_status_byname()
 *    05/16/1998 - added lvmtab checking
 *
 */

/*
 * TODO
 *
 *    05/16/1998 - support physical volumes on command line
 *                 to let the reduce take place on
 *
 */

#include <lvm_user.h>

#ifdef DEBUG
int opt_d = 0;
#endif

int main ( int argc, char **argv) {
   int c = 0;
   int c1 = 0;
   int l = 0;
   int opt_A = 1;
   int opt_f = 0;
   int opt_l = 0;
   int opt_L = 0;
   int opt_v = 0;
   int ret = 0;
   int sign = 0;
   int size_rest = 0;
   uint new_size = 0;
#ifdef DEBUG
   char *options = "A:dfh?l:L:yv";
#else
   char *options = "A:fh?l:L:yv";
#endif
   char *cmd = NULL;
   char *lv_name = NULL;
   char *vg_name = NULL;
   vg_t *vg = NULL;
   lv_t *lv = NULL;

   cmd = basename ( argv[0]);

   SUSER_CHECK;
   LVMTAB_CHECK;

   while ( ( c = getopt ( argc, argv, options)) != EOF) {
      switch ( c) {
         case 'A':
            if ( opt_A > 1) {
               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
               return 1;
            }
            if ( strcmp ( optarg, "y") == 0);
            else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
            else {
               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
                                 cmd, optarg);
               return 1;
            }
            break;

#ifdef DEBUG
         case 'd':
            if ( opt_d > 0) {
               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
               return 1;
            }
            opt_d++;
            break;
#endif

         case 'f':
            if ( opt_f > 0) {
               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
               return 1;
            }
            opt_f++;
            break;

         case 'h':
         case '?':
            printf ( "\n%s\n\n%s -- Logical volume reduce\n\n"
                     "Synopsis:\n"
                     "---------\n\n"
                     "%s\n"
                     "\t[-A y/n]\n"
#ifdef DEBUG
                     "\t[-d]\n"
#endif
                     "\t[-f]\n"
                     "\t[-h/?]\n"
                     "\t{-l [-]LogicalExtentsNumber |\n"
                     "\t -L [-]LogicalVolumeSize}\n"
                     "\t[-v]\n"
                     "\tLogicalVolumePath\n\n",
                     lvm_version, cmd, cmd);
            return 0;
            break;

         case 'l':
            if ( opt_L > 0) {
               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
               return 1;
            }
            if ( opt_l > 0) {
               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
               return 1;
            }
            sign = 0;
            if ( *optarg == '-') { sign = -1; optarg++;}
            new_size = atoi ( optarg);
            opt_l++;
            break;

         case 'L':
            if ( opt_l > 0) {
               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
               return 1;
            }
            if ( opt_L > 0) {
               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
               return 1;
            }
            sign = 0;
            if ( *optarg == '-') { sign = -1; optarg++;}
            new_size = atoi ( optarg);
            opt_L++;
            break;

         case 'v':
            if ( opt_v > 0) {
               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
               return 1;
            }
            opt_v++;
            break;

         default:
            fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                      cmd, c);
            return 1;
      }
   }


   if ( optind < argc && *argv[optind] == '-') {
      fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
      return 1;
   }

   if ( optind == argc) {
      fprintf ( stderr, "%s -- please enter a logical volume name\n\n", cmd);
      return 1;
   }
   lv_name = argv[optind];

   if ( lv_check_name ( lv_name) < 0) {
      fprintf ( stderr, "%s -- no valid logical volume name %s\n\n",
                cmd, lv_name);
      return 1;
   }

   LVM_LOCK ( 0);
   LVM_CHECK_IOP;

   vg_name = vg_name_of_lv ( lv_name);
   if ( opt_v > 0) printf ( "%s -- checking for active volume group %s\n",
                            cmd, vg_name);
   if ( vg_check_active ( vg_name) == FALSE) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " %s isn't active\n\n", cmd, vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- getting volume group status from "
                            "the VGDA in kernel\n", cmd);
   if ( vg_status ( vg_name, &vg) != 0) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " couldn't get status of %s\n\n", cmd, vg_name);
      return 1;
   }

   /* does VG exist? */
   if ( opt_v > 0) printf ( "%s -- checking existence of logical volume %s\n",
                            cmd, lv_name);
   if ( lv_check_exist ( lv_name) == FALSE) {
      fprintf ( stderr, "%s -- logical volume %s doesn't exist\n\n",
                cmd, lv_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- reading volume group data of %s "
                            "from disk(s)\n",
                            cmd, vg_name);
   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " couldn't read data of %s\n\n", cmd, vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- getting index of %s in %s\n",
                            cmd, lv_name, vg_name);
   if ( ( l = lv_get_index_by_name ( vg, lv_name)) < 0) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " couldn't get index of %s\n\n", cmd, lv_name);
      return 1;
   }

   if ( opt_L > 0 || opt_l > 0) {
      if ( opt_L > 0) new_size *= 1024 * 2;
      else            new_size *= vg->pe_size;
      if ( new_size % vg->pe_size > 0) {
         new_size += vg->pe_size;
         printf ( "%s -- rounding size to physical extend boundary\n", cmd);
      }
      new_size = new_size / vg->pe_size * vg->pe_size;
      if ( new_size >= vg->lv[l]->lv_size) {
         fprintf ( stderr, "%s -- new size larger than old one\n\n",
                   cmd);
         return 1;
      }
      if ( sign != 0) new_size = vg->lv[l]->lv_size - new_size;
      size_rest = new_size %
                  ( vg->lv[l]->lv_stripes * vg->pe_size);
      if ( size_rest != 0) {
         printf ( "%s -- rounding size %d KB to stripe boundary size ",
                  cmd, new_size / 2);
         new_size = new_size - size_rest;
         printf ( "%d KB\n", new_size / 2);
      }
   } else {
      fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- getting status of %s from VGDA in kernel\n",
                            cmd, lv_name);
   if ( lv_status_byname ( vg_name, lv_name, &lv) == 0 &&
        lv->lv_status == LV_ACTIVE) {
      printf ( "%s -- WARNING: reducing active", cmd);
      if ( lv->lv_open > 0) printf ( " and open");
      printf ( " logical volume\n"
               "%s -- THIS MAY DESTROY YOUR DATA\n", cmd);
   }

   if ( opt_v > 0) printf ( "%s -- setting up logical volume %s for reduce\n",
                            cmd, lv_name);
   if ( ( ret = lv_setup_for_reduce ( vg_name, vg, lv_name, new_size)) != 0) {
      fprintf ( stderr, "%s -- ERROR setting %s up for reduce\n\n",
                        cmd, lv_name);
      return 1;
   }

   if ( opt_f == 0) {
      c = 0;
      while ( c != 'n' && c != 'y') {
         if ( c == '\n' || c == 0)
            printf ( "%s -- do you really want to reduce %s? [y/n]: ",
                     cmd, lv_name);
         c = tolower ( getchar ());
      }
      c1 = c;
      while ( c != '\n') c = tolower ( getchar ());
      if ( c1 == 'n') {
         printf ( "%s -- %s not reduced\n\n", cmd, lv_name);
         return 0;
      }
   }

   lvm_dont_interrupt ( 0);


   /* create it in kernel */
   if ( opt_v > 0) printf ( "%s -- creating reduced logical volume "
                            "VGDA in kernel\n", cmd);
   if ( ( ret = lv_reduce ( vg, vg->lv[l], lv_name)) != 0) {
      fprintf ( stderr, "%s -- ERROR %d reducing %s in kernel\n\n", 
                cmd, ret, lv_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- storing reduced logical volume "
                            "data on disk(s)\n", cmd);
   if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of %s"
                        " on disks\n\n", 
                cmd, ret, vg_name);
      return 1;
   }


   if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
   if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, cmd, opt_v, vg) == 0 &&
        opt_A > 0) {
      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
      vg_cfgbackup ( vg_name, VG_BACKUP_DIR, cmd, opt_v, vg);
   }

   lvm_interrupt ();
   LVM_UNLOCK ( 0);

   printf ( "%s -- logical volume %s successfully reduced\n\n",
             cmd, lv_name);

   return 0;
}

==> lvm-0.4alpha.tar.gz: LVM/0.4alpha/tools/lvreduce.c <==
/*
 * Author: Heinz Mauelshagen, Germany
 *
 * April 1997
 * April-June 1998
 *
 * LVM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * LVM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 */

/*
 * Changelog
 *
 *    10/16/1997 - change lv_get_index to lv_get_index_by_name
 *    11/09/1097 - added lvmtab handling
 *    04/30/1998 - changed to lv_status_byname()
 *    05/16/1998 - added lvmtab checking
 *    06/12/1998 - enhanced checking numbers in option arguments
 *    06/27/1998 - changed lvm_tab_* calling convention
 *
 */

#include <lvm_user.h>

char *cmd = NULL;

#ifdef DEBUG
int opt_d = 0;
#endif

int main ( int argc, char **argv) {
   int c = 0;
   int c1 = 0;
   int l = 0;
   int opt_A = 1;
   int opt_f = 0;
   int opt_l = 0;
   int opt_L = 0;
   int opt_v = 0;
   int ret = 0;
   int sign = 0;
   int size_rest = 0;
   uint new_size = 0;
#ifdef DEBUG
   char *options = "A:dfh?l:L:yv";
#else
   char *options = "A:fh?l:L:yv";
#endif
   char *lv_name = NULL;
   char *vg_name = NULL;
   vg_t *vg = NULL;
   lv_t *lv = NULL;

   cmd = basename ( argv[0]);

   SUSER_CHECK;
   LVMTAB_CHECK;

   while ( ( c = getopt ( argc, argv, options)) != EOF) {
      switch ( c) {
         case 'A':
            if ( opt_A > 1) {
               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
               return 1;
            }
            if ( strcmp ( optarg, "y") == 0);
            else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
            else {
               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
                                 cmd, optarg);
               return 1;
            }
            break;

#ifdef DEBUG
         case 'd':
            if ( opt_d > 0) {
               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
               return 1;
            }
            opt_d++;
            break;
#endif

         case 'f':
            if ( opt_f > 0) {
               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
               return 1;
            }
            opt_f++;
            break;

         case 'h':
         case '?':
            printf ( "\n%s\n\n%s -- Logical Volume Reduce\n\n"
                     "Synopsis:\n"
                     "---------\n\n"
                     "%s\n"
                     "\t[-A y/n]\n"
#ifdef DEBUG
                     "\t[-d]\n"
#endif
                     "\t[-f]\n"
                     "\t[-h/?]\n"
                     "\t{-l [-]LogicalExtentsNumber |\n"
                     "\t -L [-]LogicalVolumeSize}\n"
                     "\t[-v]\n"
                     "\tLogicalVolumePath\n\n",
                     lvm_version, cmd, cmd);
            return 0;
            break;

         case 'l':
            if ( opt_L > 0) {
               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
               return 1;
            }
            if ( opt_l > 0) {
               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
               return 1;
            }
            sign = 0;
            if ( *optarg == '-') { sign = -1; optarg++;}
            if ( lvm_check_number ( optarg) == TRUE) {
               new_size = atoi ( optarg);
               opt_l++;
            } else {
               fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n\n",
                                 cmd, optarg);
               return 1;
            }
            break;

         case 'L':
            if ( opt_l > 0) {
               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
               return 1;
            }
            if ( opt_L > 0) {
               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
               return 1;
            }
            sign = 0;
            if ( *optarg == '-') { sign = -1; optarg++;}
            if ( lvm_check_number ( optarg) == TRUE) {
               new_size = atoi ( optarg);
               opt_L++;
            } else {
               fprintf ( stderr, "%s -- ERROR option L argument \"%s\"\n\n",
                                 cmd, optarg);
               return 1;
            }
            break;

         case 'v':
            if ( opt_v > 0) {
               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
               return 1;
            }
            opt_v++;
            break;

         default:
            fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                      cmd, c);
            return 1;
      }
   }


   if ( optind < argc && *argv[optind] == '-') {
      fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
      return 1;
   }

   if ( optind == argc) {
      fprintf ( stderr, "%s -- please enter a logical volume name\n\n", cmd);
      return 1;
   }
   lv_name = argv[optind];

   if ( lv_check_name ( lv_name) < 0) {
      fprintf ( stderr, "%s -- no valid logical volume name %s\n\n",
                cmd, lv_name);
      return 1;
   }

   LVM_LOCK ( 0);
   LVM_CHECK_IOP;

   vg_name = vg_name_of_lv ( lv_name);
   if ( opt_v > 0) printf ( "%s -- checking for active volume group %s\n",
                            cmd, vg_name);
   if ( vg_check_active ( vg_name) == FALSE) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " %s isn't active\n\n", cmd, vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- getting volume group status from "
                            "the VGDA in kernel\n", cmd);
   if ( vg_status ( vg_name, &vg) != 0) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " couldn't get status of %s\n\n", cmd, vg_name);
      return 1;
   }

   /* does VG exist? */
   if ( opt_v > 0) printf ( "%s -- checking existence of logical volume %s\n",
                            cmd, lv_name);
   if ( lvm_tab_lv_check_exist ( lv_name) == FALSE) {
      fprintf ( stderr, "%s -- logical volume %s doesn't exist\n\n",
                cmd, lv_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- reading volume group data of %s "
                            "from disk(s)\n",
                            cmd, vg_name);
   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " couldn't read data of %s\n\n", cmd, vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- getting index of %s in %s\n",
                            cmd, lv_name, vg_name);
   if ( ( l = lv_get_index_by_name ( vg, lv_name)) < 0) {
      fprintf ( stderr, "%s -- can't reduce logical volume:"
                        " couldn't get index of %s\n\n", cmd, lv_name);
      return 1;
   }

   if ( opt_L > 0 || opt_l > 0) {
      if ( opt_L > 0) new_size *= 1024 * 2;
      else            new_size *= vg->pe_size;
      if ( new_size % vg->pe_size > 0) {
         new_size += vg->pe_size;
         printf ( "%s -- rounding size to physical extend boundary\n", cmd);
      }
      new_size = new_size / vg->pe_size * vg->pe_size;
      if ( new_size >= vg->lv[l]->lv_size) {
         fprintf ( stderr, "%s -- new size is larger than old one\n\n",
                   cmd);
         return 1;
      }
      if ( sign != 0) new_size = vg->lv[l]->lv_size - new_size;
      size_rest = new_size %
                  ( vg->lv[l]->lv_stripes * vg->pe_size);
      if ( size_rest != 0) {
         printf ( "%s -- rounding size %d KB to stripe boundary size ",
                  cmd, new_size / 2);
         new_size = new_size - size_rest;
         printf ( "%d KB\n", new_size / 2);
      }
   } else {
      fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- getting status of %s from VGDA in kernel\n",
                            cmd, lv_name);
   if ( lv_status_byname ( vg_name, lv_name, &lv) == 0 &&
        lv->lv_status == LV_ACTIVE) {
      printf ( "%s -- WARNING: reducing active", cmd);
      if ( lv->lv_open > 0) printf ( " and open");
      printf ( " logical volume\n"
               "%s -- THIS MAY DESTROY YOUR DATA\n", cmd);
   }

   if ( opt_v > 0) printf ( "%s -- setting up logical volume %s for reduce\n",
                            cmd, lv_name);
   if ( ( ret = lv_setup_for_reduce ( vg_name, vg, lv_name, new_size)) != 0) {
      fprintf ( stderr, "%s -- ERROR setting %s up for reduce\n\n",
                        cmd, lv_name);
      return 1;
   }

   if ( opt_f == 0) {
      c = 0;
      while ( c != 'n' && c != 'y') {
         if ( c == '\n' || c == 0)
            printf ( "%s -- do you really want to reduce %s? [y/n]: ",
                     cmd, lv_name);
         c = tolower ( getchar ());
      }
      c1 = c;
      while ( c != '\n') c = tolower ( getchar ());
      if ( c1 == 'n') {
         printf ( "%s -- %s not reduced\n\n", cmd, lv_name);
         return 0;
      }
   }

   lvm_dont_interrupt ( 0);


   /* create it in kernel */
   if ( opt_v > 0) printf ( "%s -- creating reduced logical volume "
                            "VGDA in kernel\n", cmd);
   if ( ( ret = lv_reduce ( vg, vg->lv[l], lv_name)) != 0) {
      fprintf ( stderr, "%s -- ERROR %d reducing %s in kernel\n\n", 
                cmd, ret, lv_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- storing reduced logical volume "
                            "data on disk(s)\n", cmd);
   if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of %s"
                        " on disks\n\n", 
                cmd, ret, vg_name);
      return 1;
   }


   if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
   if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
        opt_A > 0) {
      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
      vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
   }

   lvm_interrupt ();
   LVM_UNLOCK ( 0);

   printf ( "%s -- logical volume %s successfully reduced\n\n",
             cmd, lv_name);

   return 0;
}
